/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.sisu.assembler.gshell.internal;

import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.directory;
import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printHeader;
import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printNumbered;

import java.io.File;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.Collection;
import java.util.List;

import org.codehaus.plexus.util.StringUtils;
import org.sonatype.gshell.command.Command;
import org.sonatype.gshell.command.CommandContext;
import org.sonatype.gshell.command.IO;
import org.sonatype.gshell.file.FileSystemAccess;
import org.sonatype.gshell.util.cli2.Option;
import org.sonatype.p2.bridge.IUIdentity;
import org.sonatype.p2.bridge.P2ProfileRegistry;
import org.sonatype.p2.bridge.ProfileTimestamp;

import com.google.inject.BindingAnnotation;
import com.google.inject.Inject;

@Command( name = "p2/list/history" )
public class ListHistoryCommand
    extends P2Command
{

    @Option( name = "p", longName = "profile" )
    private final String profile = "master";

    @Option( name = "v", longName = "verbose" )
    private boolean verbose;

    private final FileSystemAccess fileSystem;

    private final P2ProfileRegistry p2ProfileRegistry;

    private final List<ProfileTimestamp> latest;

    @Inject
    public ListHistoryCommand( final FileSystemAccess fileSystem, final P2ProfileRegistry p2ProfileRegistry,
                               @Latest final List<ProfileTimestamp> latest )
    {
        assert fileSystem != null;
        assert p2ProfileRegistry != null;
        assert latest != null;

        this.fileSystem = fileSystem;
        this.p2ProfileRegistry = p2ProfileRegistry;
        this.latest = latest;
    }

    public Object execute( final CommandContext context )
        throws Exception
    {
        assert context != null;

        final IO io = context.getIo();

        final File location = getLocation( fileSystem );

        printHeader( io, String.format( "Location %s history:", directory( location.getAbsolutePath() ) ) );

        if ( StringUtils.isBlank( profile ) )
        {
            final String[] profileIds = p2ProfileRegistry.getProfileIds( location );
            for ( final String profileId : profileIds )
            {
                io.println( profileId );
                listTimestamp( io, p2ProfileRegistry, location, profileId );
            }
        }
        else
        {
            listTimestamp( io, p2ProfileRegistry, location, profile );
        }

        io.println( "" );

        return Result.SUCCESS;
    }

    private void listTimestamp( final IO io, final P2ProfileRegistry p2ProfileRegistry, final File location,
                                final String profile )
    {
        final ProfileTimestamp[] timestamps = p2ProfileRegistry.getProfileTimestamps( location, profile );
        latest.clear();

        if ( timestamps.length == 0 )
        {
            io.println( "None" );
        }
        else
        {
            int i = 1;
            for ( final ProfileTimestamp timestamp : timestamps )
            {
                if ( verbose && i != 1 )
                {
                    io.println( "" );
                }
                printNumbered( io, i++, timestamps.length, timestamp.toString() );
                if ( verbose )
                {
                    final Collection<IUIdentity> roots = timestamp.getRoots();
                    if ( roots != null && !roots.isEmpty() )
                    {
                        for ( final IUIdentity iu : roots )
                        {
                            io.println( "\t" + iu );
                        }
                    }
                }
                latest.add( timestamp );
            }
        }
    }

    @Retention( RetentionPolicy.RUNTIME )
    @Target( { ElementType.FIELD, ElementType.PARAMETER } )
    @BindingAnnotation
    public @interface Latest
    {
    }

}
