/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.sisu.assembler.gshell.internal;

import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.directory;
import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printHeader;
import static org.sonatype.sisu.assembler.gshell.internal.PrintUtils.printNumbered;

import java.io.File;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.sonatype.gshell.command.Command;
import org.sonatype.gshell.command.CommandContext;
import org.sonatype.gshell.command.IO;
import org.sonatype.gshell.file.FileSystemAccess;
import org.sonatype.gshell.util.cli2.Option;
import org.sonatype.p2.bridge.IUIdentity;
import org.sonatype.p2.bridge.MetadataRepository;
import org.sonatype.p2.bridge.P2ProfileRegistry;

import com.google.inject.Inject;

@Command( name = "p2/list/updates" )
public class ListUpdatesCommand
    extends P2Command
{

    @Option( name = "p", longName = "profile" )
    private String profile = "master";

    @Option( name = "a", longName = "allVersions" )
    private boolean allVersions = false;

    private final FileSystemAccess fileSystem;

    private final MetadataRepository metadataRepository;

    private final P2ProfileRegistry p2ProfileRegistry;

    private final RepositoryRegistry repositoryRegistry;

    private final List<IUIdentity> latest;

    @Inject
    public ListUpdatesCommand( final FileSystemAccess fileSystem, final P2ProfileRegistry p2ProfileRegistry,
                                 final MetadataRepository metadataRepository,
                                 final RepositoryRegistry repositoryRegistry,
                                 @ListInstallableCommand.Latest final List<IUIdentity> latest )
    {
        assert fileSystem != null;
        assert metadataRepository != null;
        assert p2ProfileRegistry != null;
        assert repositoryRegistry != null;
        assert latest != null;

        this.fileSystem = fileSystem;
        this.metadataRepository = metadataRepository;
        this.p2ProfileRegistry = p2ProfileRegistry;
        this.repositoryRegistry = repositoryRegistry;
        this.latest = latest;
    }

    public Object execute( final CommandContext context )
        throws Exception
    {
        assert context != null;

        final IO io = context.getIo();

        if ( repositoryRegistry.isEmpty() )
        {
            io.error( "There should be at least one repository configured to be able to lookup available updates." );
            return Result.FAILURE;
        }

        final File location = getLocation( fileSystem );

        printHeader( io,
            String.format( "Available installable units for %s :", directory( location.getAbsolutePath() ) ) );

        final IUIdentity[] installedRoots = p2ProfileRegistry.getInstalledRoots( location, profile );

        latest.clear();

        if ( installedRoots.length == 0 )
        {
            io.println( "None" );
        }
        else
        {
            final Collection<IUIdentity> availableRoots =
                metadataRepository.getVersions( toIdSet( installedRoots ), allVersions, repositoryRegistry.getAsURI() );

            if ( availableRoots.size() == 0 )
            {
                io.println( "None" );
            }
            else
            {
                final List<IUIdentity> currentVersions = Arrays.asList( installedRoots );
                int i = 0;
                for ( final IUIdentity iu : availableRoots )
                {
                    if ( !currentVersions.contains( iu ) )
                    {
                        printNumbered( io, ++i, availableRoots.size(), iu.toString() );
                        latest.add( iu );
                    }
                }
                if ( i == 0 )
                {
                    io.println( "None" );
                }
            }
        }

        io.println( "" );

        return Result.SUCCESS;
    }

    private Set<String> toIdSet( final IUIdentity[] installedRoots )
    {
        final Set<String> roots = new HashSet<String>();
        for ( final IUIdentity iu : installedRoots )
        {
            roots.add( iu.getId() );
        }
        return roots;
    }

}
